=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-02-18 00:57:17 +0000
+++ bzrlib/repository.py	2006-02-20 04:29:08 +0000
@@ -878,6 +878,42 @@
                    RepositoryFormat6()]
 
 
+class InterRepository(object):
+    """This class represents operations taking place between two repositories.
+
+    Its instances have methods like copy_content_into and fetch, and contain
+    references to the source and target repositories these operations can be 
+    carried out on.
+
+    Often we will provide convenience methods on 'repository' which carry out
+    operations with another repository - they will always forward to
+    InterRepository.get(other).method_name(parameters).
+    """
+
+    def __init__(self, source, target):
+        """Construct a default InterRepository instance. Please use 'get'.
+        
+        Only subclasses of InterRepository should call 
+        InterRepository.__init__ - clients should call InterRepository.get
+        instead which will create an optimised InterRepository if possible.
+        """
+        self.source = source
+        self.target = target
+
+    @classmethod
+    def get(klass, repository_source, repository_target):
+        """Retrieve a InterRepository worker object for these repositories.
+
+        :param repository_source: the repository to be the 'source' member of
+                                  the InterRepository instance.
+        :param repository_target: the repository to be the 'target' member of
+                                the InterRepository instance.
+        If an optimised InterRepository worker exists it will be used otherwise
+        a default InterRepository instance will be created.
+        """
+        return InterRepository(repository_source, repository_target)
+
+
 class RepositoryTestProviderAdapter(object):
     """A tool to generate a suite testing multiple repository formats at once.
 

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2006-02-15 08:11:37 +0000
+++ bzrlib/tests/test_repository.py	2006-02-20 04:29:08 +0000
@@ -211,3 +211,21 @@
                              'w\n'
                              'W\n',
                              t.get('inventory.weave').read())
+
+
+class TestInterRepository(TestCaseWithTransport):
+
+    def test_get_default_inter_repository(self):
+        # test that the InterRepository.get(repo_a, repo_b) probes
+        # for a inter_repo class where is_compatible(repo_a, repo_b) returns
+        # true and returns a default inter_repo otherwise.
+        # This also tests that the default registered optimised interrepository
+        # classes do not barf inappropriately when a surprising repository type
+        # is handed to them.
+        dummy_a = "Repository 1."
+        dummy_b = "Repository 2."
+        inter_repo = repository.InterRepository.get(dummy_a, dummy_b)
+        self.assertEqual(repository.InterRepository,
+                         inter_repo.__class__)
+        self.assertEqual(dummy_a, inter_repo.source)
+        self.assertEqual(dummy_b, inter_repo.target)

